
class Orders:

    client = None
    response = {"state":"fail","message":"no initialized"}
    
    def set_client(self,client):
        self.client = client

    #ORDERS
    def shop_orders_search(self, access_token, fields, phrase, url_params, order = None, api='/s/MasAbrazos_AR/dw/shop/'):
        if(self.client):
            params = { 
                       
                        'query': {
                                    "text_query":  {
                                                        "fields": fields,
                                                        "search_phrase": phrase
                                                    },                                    
                                },
                        "sorts" : [{"field":"customer_name", "sort_order":"asc"}],
                        "select": "(**)"
                     }

            request_end_point = str('order_search'+url_params)
            self.client.api_service = "shop_api"
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'POST')
            ocapi_response = self.client.do_request(access_token, params)
            self.response = {"state":"ok", "message":"orders fetched","ocapi_response":ocapi_response.json(),"params":params}
        return self.response
    
    def shop_update_order_status(self,access_token, order_id, params, api='/s/MasAbrazos_AR/dw/shop/'):
        if(self.client):
            client_id = self.client.api_credentials['client_id']
            request_end_point = str('orders') + str('/') + str(order_id)  + str("?client_id=") + str(client_id)
            self.client.api_service = "shop_api"
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'PATCH')
            ocapi_response = self.client.do_request(access_token, params)
            #str('?client_id='+str(api_credentials["client_id"]))
            self.response = {"state":"ok", "message":"order updated","ocapi_response":ocapi_response.json()}
        return self.response


import time, datetime

class Products:

    client = None
    response = {"state":"fail","message":"no initialized"}
    
    def set_client(self,client):
        self.client = client

    #PRODUCTS
    def shop_product_search(self, access_token, fields, phrase, url_params, order = None, api='/s/MasAbrazos_AR/dw/shop/'):
        if(self.client):
            params = { 
                       
                        'query': {
                                    "text_query":  {
                                                        "fields": fields,
                                                        "search_phrase": phrase
                                                    },                                    
                                },
                        "expand": ['prices'],
                        "select": "(**)"
                     }

            request_end_point = str('product_search'+url_params)
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'GET')
            ocapi_response = self.client.do_request(access_token, params)
            print(ocapi_response)
            self.response = {"state":"ok", "message":"products fetched","ocapi_response":ocapi_response.json(),"params":params}
        return self.response
    
    def shop_product_search_prices(self, access_token, fields, phrase, url_params, order = None, api='/s/MasAbrazos_AR/dw/shop/'):
        if(self.client):
            params = {                        
                        'query': {
                                    "text_query":  {
                                                        "fields": fields,
                                                        "search_phrase": phrase
                                                    },                                    
                                },
                        "expand": ['prices'],
                        "select": "(**)"
                     }

            request_end_point = str('product_search/prices'+url_params)
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'GET')
            ocapi_response = self.client.do_request(access_token, params)
            print(ocapi_response)
            self.response = {"state":"ok", "message":"products fetched","ocapi_response":ocapi_response.json(),"params":params}
        return self.response

    # DATA API
    #***************************************#

    #PRODUCTS
    def product_search(self, access_token, fields, phrase, order = None, api='/s/-/dw/data/'):
        if(self.client):

            params = { 
                        'query': {
                                    "text_query":  {
                                                        "_type": "text_query",
                                                        "fields": fields,
                                                        "search_phrase": phrase
                                                    },                                    
                                },
                        "expand": ['prices'],
                        "select": "(**)"
                     }

            request_end_point = str('product_search')
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'POST')
            ocapi_response = self.client.do_request(access_token, params)
            print(ocapi_response)
            self.response = {"state":"ok", "message":"products fetched","ocapi_response":ocapi_response.json(),"params":params}
        return self.response

    
    def get_product(self, access_token, _id, api='/s/-/dw/data/'):
        if(self.client):
            params = {
                        "expand": ['availability, images, prices, variations']
             }
            request_end_point = str('products') + str('/') + str(_id)
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'GET')
            ocapi_response = self.client.do_request(access_token, params)
            self.response = {"state":"ok", "message":"products fetched","ocapi_response":ocapi_response.json()}
            return self.response
    

    def update_product_inventory_record(self, access_token,parameters, api='/s/-/dw/data/'):
        if(self.client):
            params = {
                        "allocation":   {
                                            "amount": parameters['amount']
                                        }
                      }

            #inventory_lists/MasAbrazos_AR-inventory/product_inventory_records/smart-diaper-v1-MasAbrazos-master  
            # option: set for single inventory and product id           
            if(('single_inventory' in parameters) and ('single_product_id' in parameters)):
                params = {
                            "allocation":   {
                                                "amount": parameters['amount']
                                            }
                         }
                print("updating single product")
                request_end_point = str('inventory_lists/') + str(parameters['single_inventory']) + str('/product_inventory_records/') + str(parameters['single_product_id'])
                self.client.set_api(api)
                self.client.set_request_path(request_end_point,'PATCH')
                ocapi_response = self.client.do_request(access_token, params)
                self.response = {"state":"ok", "message":"products inventory updated","ocapi_response":ocapi_response.json(),'multiple_products':False}
            else:
                products = parameters['products']
                products_inventory = []
                for product in products:
                    current_product_id = product['id']
                    current_product_amount = product['amount']
                    print("updating product " + str(current_product_id))
                    request_end_point = str('inventory_lists/') + str(parameters['single_inventory']) + str('/product_inventory_records/') + str(current_product_id)
                    self.client.set_api(api)
                    self.client.set_request_path(request_end_point,'PATCH')

                    params = {
                                "allocation":   {
                                                    "amount": current_product_amount
                                                }
                             }

                    ocapi_response = self.client.do_request(access_token, params)
                    products_inventory.append(ocapi_response.json())
                
                self.response = {"state":"ok", "message":"products inventory updated","ocapi_response":products_inventory,'multiple_products':True}                
            return self.response
    
    def get_product_inventory_record(self, access_token,parameters, api='/s/-/dw/data/'):
        if(self.client):
                request_end_point = str('inventory_lists/') +  str(parameters['single_inventory']) +  str('/product_inventory_records')
                self.client.set_api(api)
                self.client.set_request_path(request_end_point,'GET')
                ocapi_response = self.client.do_request(access_token, {})
                self.response = {"state":"ok", "message":"products inventory records fetched","ocapi_response":ocapi_response.json()}
                return self.response
    
    def update_products_price(self, access_token, customs_objects, api='/s/-/dw/data/'):
        ocapi_responses = []
        if(self.client):            

            for custom_object in customs_objects:
                now_milliseconds = int(time.time() * 1000) 
                request_end_point = str('custom_objects/PriceImportRequest') + str('/') + str(now_milliseconds)
                self.client.set_api(api)
                self.client.set_request_path(request_end_point,'PUT')
                ocapi_response = self.client.do_request(access_token, custom_object)
                ocapi_responses.append({"state":"ok", "message":"products price updated","ocapi_response":ocapi_response.json()})
            self.response = ocapi_responses 
            return self.response
    
    def cron_run_osf_cron(self, access_token, execution, api='/s/-/dw/data/'):
        if(self.client):   

            request_end_point = str('jobs') + str('/') + str(execution) + str('/') + str('executions')
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'POST')
            ocapi_response = self.client.do_request(access_token, {})
            self.response = {"state":"ok", "message":"run custom cron","ocapi_response":ocapi_response.json()}
            return self.response
    
    def cron_get_osf_job_status(self, access_token, execution, job_id, api='/s/-/dw/data/'):
        if(self.client):   

            request_end_point = str('jobs') + str('/') + str(execution) + str('/') + str('executions') + str('/') + str(job_id) 
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'GET')
            ocapi_response = self.client.do_request(access_token, {})
            self.response = {"state":"ok", "message":"cron status by job id","ocapi_response":ocapi_response.json()}
            return self.response
    
    def cron_replicate_stage_to_production(self, access_token, replication, api='/s/-/dw/data/'):
        if(self.client):   

            request_end_point = str('jobs') + str('/') + str(replication) + str('/') + str('executions')
            self.client.set_api(api)
            self.client.set_request_path(request_end_point,'POST')
            ocapi_response = self.client.do_request(access_token, {})
            self.response = {"state":"ok", "message":"cron replication","ocapi_response":ocapi_response.json()}
            return self.response
            



class Catalogs:
    client = None
    response = {"state":"fail","message":"no initialized"}
    
    def set_client(self,client):
        self.client = client

    #CATALOGS
    def get_catalogs(self, access_token, params, order = None):
        if(self.client):
            request_end_point = str('catalogs')

            if('catalog_id' in params):
                if(len(params['catalog_id'])>0):
                    request_end_point += str('/')+str(params['catalog_id'])

            self.client.set_request_path(request_end_point,'GET')
            ocapi_response = self.client.do_request(access_token)
            
            self.response = {"state":"ok", "message":"products fetched","ocapi_response":ocapi_response}
        return self.response
    
    def get_catalog_categories(self, access_token, params, order = None):
        if(self.client):
            request_end_point = str('catalogs')

            if('catalog_id' in params):
                #start base endpoint
                if(len(params['catalog_id'])>0):
                    request_end_point += str('/')+str(params['catalog_id'])
                
                #endpoint with category or all catgegories
                if('category_id' in params and len(params['catalog_id']) > 0):
                    if(len(params['category_id'])>0):
                        request_end_point += str('/categories/')+str(params['category_id'])
                    else:
                        request_end_point += str('/')+str('categories')
                   
            print(request_end_point)
            self.client.set_request_path(request_end_point,'GET')
            ocapi_response = self.client.do_request(access_token)
            self.response = {"state":"ok", "message":"products fetched","ocapi_response":ocapi_response}
        return self.response



import requests
import sys
import json
import base64
from requests.auth import HTTPBasicAuth

class Client:

    # DATA Auth
    auth_end_point = str("account.demandware.com/dw/oauth2/access_token")
    api_service = 'data_api' #shop_api
    # DATA Requests
    end_point = str("")
    end_point_request = str("")
    api_credentials = {
                            'host':None,
                            'host_staging':None,
                            'client_id':None,
                            'client_secret':None,
                            'version':None
                      }
                      
    is_secure = False
    request_method = str("POST")
    #endpoint_type = str("/s/-/dw/data/")
    #endpoint_type = str("/s/MasAbrazos_AR/dw/shop/")
    api = None

    def __init__(self, api_credentials):
        self.api_credentials = api_credentials
        self.set_base_end_point()
        self.is_secure = bool(api_credentials ['ssl_verify'])
    
    def set_api(self, api):
        self.api = api

    def do_OAuth(self):
        response = None
        basic = str(self.api_credentials['client_id']) + str(":") + str(self.api_credentials['client_secret'])
        basic = base64.b64encode(basic.encode())        

        try:  
            user = str(self.api_credentials['client_id'])
            password = str(self.api_credentials['client_secret'])
            data = []
            if(self.api_service=="data_api"):
                data =  [
                            ('grant_type', 'client_credentials'),
                        ]
            else:
                data =  [
                            ('grant_type', 'urn:demandware:params:oauth:grant-type:client-id:dwsid:dwsecuretoken'),
                        ]
                
            if(self.api_service=="data_api"):       
                headers = {
                            'Cache-Control': 'no-cache',
                            'Content-Type':'application/x-www-form-urlencoded'
                        }
            else:
                headers = {
                            'Cache-Control': 'no-cache',
                            'Content-Type':'application/x-www-form-urlencoded',
                            'Authorization':'Basic bWlsYW4ucGFuaWNAb3NmLWNvbW1lcmNlLmNvbTpDYTlsIW5hMDExOmZYKUNrUjYybVI='
                        }

            url = self.get_auth_end_point()
            print("enpoint *********")
            print(url)
            print("*****************")

            if(self.api_service=="data_api"):
                response = requests.post(url=url, data=data, headers=headers, auth=HTTPBasicAuth(user, password))
            else:
                response = requests.post(url=url, data=data, headers=headers)

            response = json.loads(response.content.decode("utf-8"))
            #with open('/home/rockscripts/Documentos/odoo/salesforce/service/ocapi/log.json', 'w') as outfile:
            #    json.dump(response, outfile) 

        except Exception as e:
            exc_traceback = sys.exc_info()
            print(repr(e))
            #with open('/home/rockscripts/Documentos/odoo/salesforce/service/ocapi/log.json', 'w') as outfile:
            #    json.dump(getattr(e, 'message', repr(e))+" ON LINE "+format(sys.exc_info()[-1].tb_lineno), outfile)            

        return response

    def do_request(self, access_token, parameters = {}):
        response = None

        headers = {
                    "Authorization": "Bearer " + str(access_token),
                    "Accept":"application/json",
                    "Content-Type":"application/json",
                  }   

        #print(json.dumps(parameters))  
        print(self.get_end_point())
        req = requests.Request(self.request_method,self.get_end_point(),headers=headers, json=parameters)
        prepared = req.prepare()
        s = requests.Session()

        try:
            response = s.send(prepared,verify=self.is_secure)   
            #print(response.content)
            
        except Exception as e:
            exc_traceback = sys.exc_info()
            print(repr(e))         

        return response
    
    def set_base_end_point(self):
        self.end_point = self.api_credentials['host']
        return self.end_point
    
    def set_request_path(self, request_end_point, method):
        self.end_point_request =  self.api  + str(self.api_credentials["version"]) + str("/") + str(request_end_point)
        
        self.request_method = method

    def get_end_point(self):
        if (self.is_secure==False):
            return str("http://") +  str(self.end_point) + str(self.end_point_request)
        else:
            return str("https://") +  str(self.end_point) + str(self.end_point_request)
    
    def get_auth_end_point(self):
        if(self.api_service=="shop_api"):    
            self.auth_end_point = self.api_credentials["host"]+ str("/dw/oauth2/access_token") + str("?client_id=") + str(self.api_credentials["client_id"])

        if (self.is_secure==False):
            self.auth_end_point = str("http://") + self.auth_end_point
        else:
            self.auth_end_point = str("https://") + self.auth_end_point

        print("CLIENT")
        print(self.auth_end_point)
        return self.auth_end_point


import setuptools
from sys import version

if version < '2.2.3':
	from distutils.dist import DistributionMetadata
	DistributionMetadata.classifiers = None
	DistributionMetadata.download_url = None
	
from distutils.core import setup


setuptools.setup(
                    name='sfws',
                    version='2.0.1',
                    author='@rockscripts',
                    author_email='rockscripts@gmail.com',
                    description='SF Company',
                    long_description="SF Company",
                    install_requires=[],
                    platforms='any',
                    url='http://odoo.com',
                    packages=['sfws'],
                    python_requires='>=2.7.*',
                    classifiers=[
                                    'License :: OSI Approved :: BSD License',
                                    'Programming Language :: Python :: 3',
                                    'Topic :: Software Development :: Libraries :: Python Modules',
                                ],
                )


import json
import sys
from . client import Client
from . catalogs import Catalogs
from . products import Products
from . orders import Orders

class Service:
    client = None
    catalogs = None   
    api_credentials = None

    def start_client(self, api_credentials, api_service="data_api"):
        self.client = Client(api_credentials)
        self.client.api_service = api_service
        self.api_credentials = api_credentials
        

        self.catalogs = Catalogs()
        self.catalogs.set_client(self.client)

        self.products = Products()
        self.products.set_client(self.client)

        self.orders = Orders()
        self.orders.set_client(self.client)

    def connect(self):
        if(self.client):
            ocapi_response = self.client.do_OAuth()

            if('access_token' in ocapi_response):
                self.response = {"state":"ok", "message":"connected","ocapi_response":ocapi_response}
            else:
                self.response = {"state":"fail", "message":ocapi_response['error_description'],"ocapi_response":ocapi_response}

        return self.response
    
    
    #CATALOGS
    def get_catalogs(self, access_token, params, order = None):
        return self.catalogs.get_catalogs(access_token, params, order)
    
    def get_catalog_categories(self, access_token, params, order = None):
        return self.catalogs.get_catalog_categories(access_token, params, order)

    #DATA PRODUCTS
    def product_search(self, access_token, fields, phrase, order = None):
        return self.products.product_search(access_token, fields, phrase, order)

    def update_product_inventory_record(self, access_token,params):
        return self.products.update_product_inventory_record(access_token, params)

    def get_product_inventory_record(self, access_token,params):
        return self.products.get_product_inventory_record(access_token, params)
    
    def update_products_price(self, access_token, customs_objects):
        return self.products.update_products_price(access_token, customs_objects)
    
    def cron_run_osf_cron(self, access_token, execution):
        return self.products.cron_run_osf_cron(access_token, execution)
    
    def cron_get_osf_job_status(self, access_token, execution, job_id):
        return self.products.cron_get_osf_job_status(access_token, execution, job_id)
    
    def cron_replicate_stage_to_production(self, access_token, replication):
        return self.products.cron_replicate_stage_to_production(access_token, replication)
    
    #GET PRODUCTS
    def get_product(self, access_token, _id):
        return self.products.get_product(access_token, _id)

    #SHOP PRODUCTS
    def shop_product_search(self, access_token, fields, phrase, api, order = None):
        return self.products.shop_product_search(access_token, fields, phrase, api, order)
    
    #PRICES
    def shop_product_search_prices(self, access_token, fields, phrase, api, order = None):
        return self.products.shop_product_search_prices(access_token, fields, phrase, api, order)

    
    #ORDERS
    def shop_orders_search(self, access_token, fields, phrase, api, order = None):
        return self.orders.shop_orders_search(access_token, fields, phrase, api, order)
    
    def shop_update_order_status(self,access_token, order_id, new_status, api):
        return self.orders.shop_update_order_status(access_token, order_id, new_status, api)    